-- Daily Global Check
-- by Jadya
-- EU-Well of Eternity

local addonName, addonTable = ...
local L = addonTable.L

local lower, tsort = string.lower, table.sort

-- string colors
local AddColor = addonTable.AddColor

local constructors = DailyGlobalCheck.Constructors
local lists = DailyGlobalCheck.ListFunctions
local editframe = DailyGlobalCheck.EditPanel
local pages = DailyGlobalCheck.PagesFunctions
local menus = DailyGlobalCheck.MenuFunctions
local help = DailyGlobalCheck.HelpFunctions
local maps = DailyGlobalCheck.MapFunctions

local tx_arrowl = "Interface\\icons\\misc_arrowleft"
local tx_arrowr = "Interface\\icons\\misc_arrowright"
local tx_addpage = "Interface\\icons\\Spell_ChargePositive"
local tx_rempage = "Interface\\icons\\Spell_ChargeNegative"
local tx_addquest = "Interface\\icons\\PetBattle_Speed"
local tx_addgroup = "Interface\\icons\\INV_Misc_GroupNeedMore"
local tx_moveup = "Interface\\icons\\misc_arrowlup"
local tx_movedown = "Interface\\icons\\misc_arrowdown"
local tx_deleteselected = "Interface\\COMMON\\VOICECHAT-MUTED"
local tx_columnsort = "Interface\\Buttons\\UI-MicroStream-Green"

local editframe_list
local opened_list, selected, selected_max
local f_preview, f_quests, btn_delete_sel, edt_rename_group

local selected_limit = 42

local HelpPlate = {
	FramePos = { x = 0, y = -60 },
	FrameSize = { width = 1000, height = 690 },
	[1] = { ButtonPos = { x = 650, y = -50 },  HighLightBox = { x = 620, y = -33, width = 360, height = 80 },	 ToolTipDir = "UP",  ToolTipText = L["help_edt_6"] },
    [2] = { ButtonPos = { x = 530, y = -25 }, HighLightBox = { x = 10, y = -33, width = 560, height = 30 },  ToolTipDir = "RIGHT",   ToolTipText = L["help_edt_7"] },
	[3] = { ButtonPos = { x = 250, y = -250 }, HighLightBox = { x = 10, y = -65, width = 560, height = 420 }, ToolTipDir = "DOWN",   ToolTipText = L["help_edt_8"] },
	[4] = { ButtonPos = { x = 450, y = -600},  HighLightBox = { x = 10, y = -490, width = 560, height = 190 },  ToolTipDir = "DOWN", ToolTipText = L["help_edt_9"] },
	[5] = { ButtonPos = { x = 740, y = -400 },  HighLightBox = { x = 620, y = -115, width = 360, height = 500 },  ToolTipDir = "DOWN",  ToolTipText = L["help_edt_10"] },
	[6] = { ButtonPos = { x = 570, y = -400 },  HighLightBox = { x = 580, y = -33, width = 30, height = 647 },  ToolTipDir = "LEFT",  ToolTipText = L["help_edt_11"] },
	[7] = { ButtonPos = { x = 650, y = -620 },  HighLightBox = { x = 620, y = -620, width = 360, height = 60 },  ToolTipDir = "UP",  ToolTipText = L["help_edt_12"] },
	[8] = { ButtonPos = { x = 160, y = 10 },  HighLightBox = { x = 160, y = 10, width = 410, height = 40 },  ToolTipDir = "UP",  ToolTipText = L["help_edt_13"] },
}

local function ToggleTutorial()
 if not help:isShowing(HelpPlate) then
  help:Show(HelpPlate, editframe_list, DGC_EFLHelpButton)
 else
  help:Hide()
 end
end

local function clearSelection()
 edt_rename_group:Hide()
 edt_rename_group:ClearFocus()
 selected = nil
end

local function pageLimit()
 local i = 0
 for pindex,p in pairs(opened_list["Order"][pages:Curr(opened_list)]) do
  i = i + #p + 1
 end

 return i >= selected_limit
end

-- returns page index and element index of the current selection
local function getselectedindexes()
 local i = 0
 for pindex,p in pairs(opened_list["Order"][pages:Curr(opened_list)]) do
  for index,questID in pairs(p) do
   i = i + 1
   if i == selected then
    return pindex, index
   end
  end
  i = i + 1
 end
end

local function drawPreview()
 local s = ""
 local i = 0

 btn_delete_sel:Hide()

 for _,p in pairs(opened_list["Order"][pages:Curr(opened_list)]) do
  for index,questID in pairs(p) do
   i = i + 1
   
   if i == selected then
    btn_delete_sel:SetPoint("TOPLEFT", f_preview, "TOPLEFT", 0, -42 - i * 12)
    btn_delete_sel:Show()
	
    local _, index = getselectedindexes()
	if index == 1 then
	 edt_rename_group:SetPoint("TOPLEFT", f_preview, "TOPLEFT", 15, -42 - i * 12)
	else
	 edt_rename_group:Hide()
	end
   end
   
   if index == 1 then -- header
    s = i == selected and s.."|cff00FF00>"..questID.."|r\n" or s..AddColor(questID, "GOLD").."\n"
   else
    local data = DailyGlobalCheck:getQData(questID)
    if data then
     local name = data[2]
     s = i == selected and s.."  |cff00FF00>"..name.."|r\n" or s..AddColor(" "..name, "NAME_COLOR").."\n"
    else
     s = s.."<no data>\n"
    end
   end
  end
  s = s.."\n"
  i = i + 1
 end
 
 selected_max = i - 1
 
 f_preview.elements:SetText(s)
 f_preview.page:SetText(pages:GetString(opened_list))
 f_preview:UpdateArrows()
end

---------------------------------------------------
-------------- quest edit section -----------------
local function quest_edit_refresh()

 if not f_quests.selected then
  editframe_list.btnremove:Hide()
  return
 end

 local sel = f_quests.selected - HybridScrollFrame_GetOffset(f_quests)
 local questID = f_quests.buttons[sel].qID
 local data = DailyGlobalCheck:getQData(questID)
 
 if data then 
  -- quest ID
  editframe_list.edtQID:SetText(questID)
  -- name
  editframe_list.edtNAME:SetText(data[2] or L["namemissing"])
  -- prefix
  editframe_list.edtPREF:SetText(data[3] or "")
  -- suffix
  editframe_list.edtSUFF:SetText(data[4] or "")
  -- zone text
  UIDropDownMenu_SetText(editframe_list.ddZONE, data[1] or "")
  -- map data
  if data[5] then
   -- default map
   local zID = data[6]
   editframe_list.ddZONE.zID = zID
   editframe_list.edtCoordX:SetText(data[5][zID][1])
   editframe_list.edtCoordY:SetText(data[5][zID][2])
  else
   editframe_list.ddZONE.zID = nil
   editframe_list.edtCoordX:SetText("0")
   editframe_list.edtCoordY:SetText("0")
  end
 end

 -- fixed quest (imported from a plugin)
 if not data or data[11] then 
  editframe_list.btnremove:Hide()
 else
  editframe_list.btnremove:Show()
 end
end

local function quest_edit_apply()
 local questID = editframe_list.edtQID:GetText()
 local name = editframe_list.edtNAME:GetText()
 local pref = editframe_list.edtPREF:GetText()
 local suff = editframe_list.edtSUFF:GetText()
 local sel = f_quests.selected and (f_quests.selected - HybridScrollFrame_GetOffset(f_quests)) or -1
 --local questID = f_quests.buttons[sel] and f_quests.buttons[sel].qID
 local questID = editframe_list.edtQID:GetText()
 
 if questID == "" then return end
 
 if tonumber(questID) then
  questID = tonumber(questID)
 end
 local data = DailyGlobalCheck:getQData(questID)
 
 if not data then
  DGC_CustomLists.QuestsData[questID] = {}
  data = DGC_CustomLists.QuestsData[questID]
 end
 
 -- can't edit fixed quests
 if data[11] then return end
 
 if name == "" then name = "<no name>" end

 data[1] = UIDropDownMenu_GetText(editframe_list.ddZONE)
 data[2] = name
 data[3] = pref
 data[4] = suff
 if editframe_list.ddZONE.zID and editframe_list.ddZONE.zID > -1 then
  local zID = editframe_list.ddZONE.zID
  local x   = tonumber(editframe_list.edtCoordX:GetText()) or 0
  local y   = tonumber(editframe_list.edtCoordY:GetText()) or 0
  data[5] = {[zID] = {x, y}}
  data[6] = zID
  maps:resolveCoords(DGC_CustomLists.QuestsData[questID])
 end

 drawPreview()
 f_quests:update()
end

local function quest_edit_remove()
 if not f_quests.selected then return end

 local sel = f_quests.selected - HybridScrollFrame_GetOffset(f_quests)
 local questID = f_quests.buttons[sel].qID
 
 DGC_CustomLists.QuestsData[questID] = nil
 
 DailyGlobalCheck:foreachLists(function(k,v)
  local pp, pi, i = lists:ContainsQuest(v, questID)
 
  if pp then
   table.remove(v["Order"][pp][pi], i)
  end
 end, false)

 f_quests.selected = nil
 quest_edit_refresh()
 drawPreview()
 f_quests:update()
end

function editframe:quest_edit_SelectZone(zID)
 if zID == -1 then
  UIDropDownMenu_SetText(editframe_list.ddZONE, NONE)
  editframe_list.ddZONE.zID = nil
 else
  UIDropDownMenu_SetText(editframe_list.ddZONE, GetMapNameByID(zID))
  editframe_list.ddZONE.zID = zID
 end
end
-------------- quest edit end -------------------
-------------------------------------------------

-------------------------------------------------
-------------- preview edit ---------------------
local function delete_element()
 local pi, i = getselectedindexes()

 if not pi or not i then return end
 
 local t = opened_list["Order"][pages:Curr(opened_list)]
 
 if i == 1 then -- delete the selected group
  if #t <= 1 then return end
  edt_rename_group:Hide()
  edt_rename_group:ClearFocus()
  table.remove(t, pi)
 else -- delete the selected quest
  table.remove(t[pi], i)
 end
 
 f_quests:update()
 drawPreview()
 quest_edit_refresh()
end

local function rename_selected_group()
  local pi = getselectedindexes()
  if not pi then return end
  local t = opened_list["Order"][pages:Curr(opened_list)]
  t[pi][1] = edt_rename_group:GetText()
  drawPreview()
  edt_rename_group:Hide()
 end

local function move_element(self)
 local pi, i = getselectedindexes()
 
 if not pi or not i then return end

 local t = opened_list["Order"][pages:Curr(opened_list)]
 
 local up = self.tag == 0
 local n = up and -1 or 1

 if (up and i == 1 and pi > 1) or
    (not up and i == 1 and pi < #t) then -- it's a header, move the entire group
  local tmp = t[pi]
  t[pi] = t[pi + n]
  t[pi + n] = tmp
  selected = selected + n + (up and -#t[pi] or #t[pi])
 elseif (up and i > 2) or
        (not up and i > 1 and i < #t[pi]) then -- it's a quest, move it inside its group
  local tmp = t[pi][i]
  t[pi][i] = t[pi][i + n]
  t[pi][i + n] = tmp
  selected = selected + n
 elseif (up and i == 2 and pi > 1) or
        (not up and i == #t[pi] and pi < #t) then -- it's the first or the last quest, move it into another group if possible
  local Idest = up and #t[pi + n] + 1 or 2
  local tmp   = t[pi][i]
  table.remove(t[pi], i)
  table.insert(t[pi + n], Idest, tmp)
  -- keep the same quest selected considering the space between groups
  selected = selected + n * 2
 end

 drawPreview()
end

local function add_group()
 if pageLimit() then return end

 local g = {L["new_group"]}
  
 table.insert(opened_list["Order"][pages:Curr(opened_list)], g)

 drawPreview()
end

local function add_quest()

 if not f_quests.selected or pageLimit() then return end

 local sel = f_quests.selected - HybridScrollFrame_GetOffset(f_quests)
 
 local questID = f_quests.buttons[sel].qID

 if not questID then return end

 local n = tonumber(questID)
 questID = n or questID

 if not questID or lists:ContainsQuest(opened_list, questID) then return end

 local pi,i = getselectedindexes()
 
 if not pi then pi = 1 end
 if not i then i = 1 end

 local t = opened_list["Order"][pages:Curr(opened_list)][pi]

 if i < #t then
  table.insert(t, i + 1, questID)
 else
  t[i + 1] = questID
 end

 selected = (selected or 1) + 1
 drawPreview()
 f_quests:update()
end

local function new_page()
 pages:New(opened_list)
 pages:Move(opened_list, pages:Max() - pages:Curr())
 drawPreview()
 f_quests:update()
end

local function del_page()
 pages:Remove(opened_list)
 drawPreview()
 f_quests:update()
end

------------- preview edit end ------------------
-------------------------------------------------

function editframe:EditList(v)
 opened_list = v
 editframe_list.edttitle:SetText(v["Title"])
 drawPreview()
 
 editframe:Hide()
 editframe_list:Show()
end

local function sfQuestsUpdate()

 local i_quests = {}
 local i = 1
 DailyGlobalCheck:foreachQData(function(k,v)
  local q = DailyGlobalCheck:getQData(k)
  if not q[12] and (not q[11] or (q[11] and f_quests.filters[q[11]])) and
     (not f_quests.textfilter or
     tostring(k):find(f_quests.textfilter) or
     (q[1] and lower(q[1]):find(f_quests.textfilter)) or 
     (q[2] and lower(q[2]):find(f_quests.textfilter))) then
   i_quests[i] = k
   i = i + 1
  end
 end)

 local function sortTable(a, b)
  local a1, b1
  if f_quests.sort == -1 then
   a1 = a
   b1 = b
  else
   local aData = DailyGlobalCheck:getQData(a)
   local bData = DailyGlobalCheck:getQData(b)
  
   a1 = aData[f_quests.sort]
   b1 = bData[f_quests.sort]
  end
  
  a1 = lower(tostring(a1))
  b1 = lower(tostring(b1))
  if f_quests.sortDir then
   return a1 < b1
  else
   return a1 > b1
  end
 end

 tsort(i_quests, sortTable)
 
 local height = 20 -- height of a row
 local offset = HybridScrollFrame_GetOffset(f_quests)
 local buttons = f_quests.buttons

 local length = #i_quests
 editframe_list.btnremove:Hide()
 
 for i=1, #buttons do
  local index = offset + i
  if index <= length then
   local col = lists:ContainsQuest(opened_list, i_quests[index]) and "|cff777777" or "|cffFFFFFF"
   local data = DailyGlobalCheck:getQData(i_quests[index])
   
   if index == f_quests.selected then
    buttons[i]:LockHighlight()
	if not data[11] then
     editframe_list.btnremove:SetPoint("RIGHT", buttons[i], "LEFT")
 	 editframe_list.btnremove:Show()
	end
   else
    buttons[i]:UnlockHighlight()
   end
   
   if data[11] then
    local _, l = lists:Find(data[11], false)
    buttons[i].icon2:SetTexture(l and l["Icon"])
    buttons[i].icon2:Show()
   else
    buttons[i].icon2:Hide()
   end
   if data[7] == "Q" then
    buttons[i].icon:SetTexCoord(0.125, 0.256, 0.125, 0.256)
   elseif data[7] == "W" then
    buttons[i].icon:SetTexCoord(0.625, 0.75, 0.125, 0.256)
   else
    buttons[i].icon:SetTexCoord(0.375, 0.5, 0.125, 0.256)
   end
   buttons[i].questID:SetText(col..(i_quests[index] or "").."|r")
   buttons[i].zone:SetText(col..(data[1] or "").."|r")
   buttons[i].name:SetText(col..(data[2] or "").."|r")
   buttons[i].qID = i_quests[index]
   buttons[i]:Show()
  else
   buttons[i]:Hide()
  end
  HybridScrollFrame_Update(f_quests, height * length, height)
 end
end

local function sfButtonSelected(self)
 local sel = f_quests.selected - HybridScrollFrame_GetOffset(f_quests)
 editframe_list.btnremove:SetPoint("RIGHT", f_quests.buttons[sel], "LEFT")
 quest_edit_refresh()
end

local function sfQuestsBtnClick(self, button)
 if not f_quests.selected then return end
 local sel = f_quests.selected - HybridScrollFrame_GetOffset(f_quests)
 if sel == self.index then
  add_quest()
 end
end

local function sfSetFilter(self)
 f_quests.textfilter = lower(self:GetText())
 f_quests.selected = nil
 sfQuestsUpdate()
 quest_edit_refresh()
end

local function sfColumnClick(self)
 if f_quests.sort == self.tag then
  f_quests.sortDir = not f_quests.sortDir
 else
  f_quests.sortDir = true
  f_quests.sort = self.tag
 end
 f_quests.selected = nil
 sfQuestsUpdate()
 editframe_list.sort_arrow:SetParent(self)
 editframe_list.sort_arrow:SetPoint("TOPLEFT", self, "TOPLEFT", 0, 15)
 editframe_list.sort_arrow:SetRotation(f_quests.sortDir and 0 or PI)
 quest_edit_refresh()
end

function editframe:CreateListEditFrame()
 editframe_list = constructors:Panel(UIParent, "DGC_ListEditPanel", 1000, 800, true)

 --------------------
 -- Quests listbox --
 f = constructors:ListBox(editframe_list, "DGC_EFQuestsScrollFrame", 525, 430, "DGCListButtonTemplate", sfQuestsUpdate, sfQuestsBtnClick, sfButtonSelected)
 f:SetPoint("TOPLEFT", editframe_list, "TOPLEFT", 20, -120)
 f:SetPoint("BOTTOMLEFT", editframe_list, "BOTTOMLEFT", 20, 260)
 f.columns = {}
 f.filters = {}
 for _,v in pairs(DailyGlobalCheck.Lists) do
  f.filters[v["Title"]] = true
 end
 f_quests = f
 
 -- Search filter
 f = constructors:EditBox(editframe_list, SEARCH)
 f:SetPoint("BOTTOMRIGHT", f_quests, "TOPRIGHT", 0, 30)
 f:SetWidth(200)
 f:SetScript("OnTextChanged", sfSetFilter)
 editframe_list.edtSearch = f
 
 -- Dropdown filter
 f = CreateFrame("Button", "DGCFilterEditDropDown", editframe_list, "UIDropDownMenuTemplate")
 f:SetPoint("RIGHT", editframe_list.edtSearch, "LEFT", -10, 0)
 UIDropDownMenu_SetWidth(f, 100)
 UIDropDownMenu_SetText(f, FILTER)
 UIDropDownMenu_Initialize(f, menus.Init_QuestsFilterDropdown)
 editframe_list.ddFilter = f
 
 -- Columns
 f = editframe_list:CreateTexture()
 f:SetSize(32, 32)
 f:SetTexture(tx_columnsort)
 editframe_list.sort_arrow = f
 
-- List Icon
 f = constructors:largebuttontemplate(f_quests, "L", "")
 f:SetBackdropColor(0,0,0,0.9)
 f:SetPoint("TOPLEFT", f_quests, "TOPLEFT", -2, 25)
 f:SetWidth(20)
 f.tag = 11
 f:SetScript("OnMouseUp", sfColumnClick)
 f_quests.columns[1] = f
 
 -- Quest Type
 f = constructors:largebuttontemplate(f_quests, TYPE, "")
 f:SetBackdropColor(0,0,0,0.9)
 f:SetPoint("LEFT", f_quests.columns[1], "RIGHT", 1)
 f:SetWidth(40)
 f.tag = 7
 f:SetScript("OnMouseUp", sfColumnClick)
 f_quests.columns[2] = f

 -- QuestID
 f = constructors:largebuttontemplate(f_quests, L["b_questid"], "")
 f:SetBackdropColor(0,0,0,0.9)
 f:SetPoint("LEFT", f_quests.columns[2], "RIGHT", 1)
 f:SetWidth(75)
 f.tag = -1
 f:SetScript("OnMouseUp", sfColumnClick)
 f_quests.columns[3] = f
 
 -- Zone
 f = constructors:largebuttontemplate(f_quests, LFG_TYPE_ZONE, "")
 f:SetBackdropColor(0,0,0,0.9)
 f:SetPoint("LEFT", f_quests.columns[3], "RIGHT", 1)
 f:SetWidth(175)
 f.tag = 1
 f:SetScript("OnMouseUp", sfColumnClick)
 f_quests.columns[4] = f
 
 -- Name
 f = constructors:largebuttontemplate(f_quests, CALENDAR_EVENT_NAME, "")
 f:SetBackdropColor(0,0,0,0.9)
 f:SetPoint("LEFT", f_quests.columns[4], "RIGHT", 1)
 f:SetWidth(242)
 f.tag = 2
 f:SetScript("OnMouseUp", sfColumnClick)
 f_quests.columns[5] = f
 
 --------------------
 -- Quests Editor -- 
 -- container
 f = CreateFrame("Frame", nil, editframe_list)
 editframe_list.edtQContainer = f
 
 local container = editframe_list.edtQContainer
 
 -- quest ID
 f = constructors:EditBox(container, L["b_questid"])
 f:SetPoint("TOPLEFT", f_quests, "BOTTOMLEFT", 0, -30)
 f:SetWidth(100)
 f:SetScript("OnTextChanged", function(self)
  local s = self:GetText()
  if DailyGlobalCheck:getQData(tonumber(s) or s) then
   editframe_list.btnapply:SetText(APPLY)
  else
   editframe_list.btnapply:SetText(ADD)
  end
 end)
 editframe_list.edtQID = f
 
 -- name
 f = constructors:EditBox(container, NAME)
 f:SetPoint("LEFT", editframe_list.edtQID, "RIGHT", 15, 0)
 f:SetWidth(200)
 editframe_list.edtNAME = f

 -- zone
 f = CreateFrame("Button", "DGCZoneEditDropDown", container, "UIDropDownMenuTemplate")
 f:SetPoint("LEFT", editframe_list.edtNAME, "RIGHT", 15, 0)
 f.lbl = f:CreateFontString()
 f.lbl:SetFont("Fonts\\ARIALN.ttf", 14, "")
 f.lbl:SetPoint("BOTTOMLEFT", f, "TOPLEFT", 15, 0) 
 f.lbl:SetText(ZONE)
 UIDropDownMenu_SetWidth(f, 150)
 UIDropDownMenu_Initialize(f, menus.Init_ZonesDropdown)
 editframe_list.ddZONE = f
 
 -- Coordinates X
 f = constructors:EditBox(container, "X")
 f:SetPoint("TOPLEFT", editframe_list.ddZONE, "BOTTOMLEFT", 15, -15)
 f:SetWidth(60)
 editframe_list.edtCoordX = f
 
 -- Coordinates Y
 f = constructors:EditBox(container, "Y")
 f:SetPoint("LEFT", editframe_list.edtCoordX, "RIGHT", 5, 0)
 f:SetWidth(60)
 editframe_list.edtCoordY = f
 
 -- prefix
 f = constructors:EditBox(container, L["b_prefix"])
 f:SetPoint("TOPLEFT", editframe_list.edtQID, "BOTTOMLEFT", 0, -25)
 f:SetWidth(100)
 editframe_list.edtPREF = f
 
 -- suffix
 f = constructors:EditBox(container, L["b_suffix"])
 f:SetPoint("LEFT", editframe_list.edtPREF, "RIGHT", 15, 0)
 f:SetWidth(100)
 editframe_list.edtSUFF = f

 -- apply new quest button
 f = constructors:CreateListButton(container)
 f:SetText(ADD)
 f:SetSize(75, 50)
 f:SetPoint("TOPLEFT", editframe_list.edtPREF, "BOTTOMLEFT", 0, -20)
 f.clickfunc = quest_edit_apply
 editframe_list.btnapply = f
 -- Quests Editor End --
 -----------------------

 -------------------
 -- Preview Frame --
 f = CreateFrame("Frame", "DGC_ListPreviewFrame", editframe_list)
 f:SetPoint("TOPLEFT", editframe_list, "TOPRIGHT", -370, -120)
 f:SetPoint("BOTTOMRIGHT", editframe_list, "BOTTOMRIGHT", -20, 60)
 constructors:SetupFrame(f)
 f:SetFrameStrata(editframe_list:GetFrameStrata())
 f.elements = f:CreateFontString(nil, "OVERLAY", "GameTooltipText")
 f.elements:SetPoint("TOPLEFT", f, "TOPLEFT", 20, -50)
 f_preview = f

 -- element selection
 f_preview.GetCursorElement = function(self)
  local x,y = GetCursorPosition()
  local sc = UIParent:GetEffectiveScale()
  local px, py = f_preview:GetLeft() * sc, f_preview:GetTop() * sc
  x = x - px
  y = py - y

  if x < 0 or x > f_preview:GetWidth() then return end
  if y < 32 then return end
  y = y - 32

  return math.floor(y / 9)
 end

 f_preview:SetScript("OnMouseUp", function(self)
  local new_sel = f_preview:GetCursorElement()
  local pi, i = getselectedindexes()

  if i == 1 and selected == new_sel then
   edt_rename_group:Show()
   edt_rename_group:SetText(opened_list["Order"][pages:Curr(opened_list)][pi][1])
   edt_rename_group:SetFocus()
  elseif edt_rename_group:IsVisible() then
   rename_selected_group()
  end

  selected = new_sel
  if selected and selected > selected_max then selected = nil end
  drawPreview()
 end)
 
 -- Rename group
 f = constructors:EditBox(f_preview, nil, rename_selected_group)
 f:SetWidth(300)
 f:SetHeight(20)
 f:SetBackdropColor(0,0,0,1)
 f:Hide()
 edt_rename_group = f
 
 -- Title
 local f = constructors:EditBox(f_preview, LFG_LIST_TITLE)
 f:SetScript("OnEditFocusLost", function(self)
  local newtitle = self:GetText()
  if lists:Find(newtitle, false) then
   self:SetText(opened_list["Title"])
  else
   lists:Rename(opened_list, newtitle)
  end
 end)
 f:SetPoint("TOP", f_preview, "TOP", 0, -20)
 f:SetWidth(200)
 editframe_list.edttitle = f
 
 editframe_list:SetScript("OnHide", function() help:Hide() end)

 -- Help Button
 f = CreateFrame("Button", "DGC_EFLHelpButton", editframe_list, "MainHelpPlateButton")
 f:SetPoint("TOPLEFT")
 f:SetScript("OnClick", function()
  ToggleTutorial()
 end)
 
 -- Remove Quest
 f = constructors:CreateStaticBtn(editframe_list, tx_deleteselected, {"TOPLEFT", f_quests, "TOPRIGHT", 40, -20}, {REMOVE.." "..BATTLE_PET_SOURCE_2})
 f:SetScript("OnMouseUp", quest_edit_remove)
 f:SetSize(20,20)
 f:ClearAllPoints()
 editframe_list.btnremove = f
 
 -- Add group
 f = constructors:CreateStaticBtn(editframe_list, tx_addgroup, {"LEFT", f_quests, "RIGHT", 40, 20}, {L["new_group"]})
 f:SetScript("OnMouseUp", add_group)

 -- Add quest
 f = constructors:CreateStaticBtn(editframe_list, tx_addquest, {"LEFT", f_quests, "RIGHT", 40, -20}, {L["add_quest"]})
 f:SetScript("OnMouseUp", add_quest)
 
 -- Delete selected quest/header
 f = constructors:CreateStaticBtn(f_preview, tx_deleteselected, {"TOPLEFT"}, {DELETE})
 f:SetSize(15, 15)
 f:SetScript("OnMouseUp", delete_element)
 btn_delete_sel = f
 
 -- Move quest/header up and down
 f = constructors:CreateStaticBtn(f_preview, tx_moveup, {"LEFT", f_preview, "RIGHT", -40, 20}, {L["move_up"]})
 f.tag = 0
 f:SetScript("OnMouseUp", move_element)

 f = constructors:CreateStaticBtn(f_preview, tx_movedown, {"LEFT", f_preview, "RIGHT", -40, -20}, {L["move_down"]})
 f.tag = 1
 f:SetScript("OnMouseUp", move_element)
 
 
 -- Pages
 -- page title
 f_preview.page = f:CreateFontString()
 f_preview.page:SetFont("Fonts\\ARIALN.ttf", 13, "")
 f_preview.page:SetPoint("BOTTOM", f_preview, "BOTTOM", 0, 45)

 -- prev page button
 f = constructors:CreateStaticBtn(f_preview, tx_arrowl, {"BOTTOMLEFT", f_preview, "BOTTOMLEFT", 5, 10}, {L["prevpage"]})
 f:SetScript("OnMouseUp", function(self)
   if pages:Move(opened_list, -1) then
    clearSelection()
    drawPreview()
   end
  end)
 f_preview.Lbtn = f
 
 -- next page button
 f = constructors:CreateStaticBtn(f_preview, tx_arrowr, {"BOTTOMRIGHT", f_preview, "BOTTOMRIGHT", -5, 10}, {L["nextpage"]})
 f:SetScript("OnMouseUp", function(self)
   if pages:Move(opened_list, 1) then
    clearSelection()
    drawPreview()
   end
  end)
 f_preview.Rbtn = f
 
 f_preview.UpdateArrows = function(self)
      
   if pages:Curr(opened_list) == 1 then
    self.Lbtn.tex:SetVertexColor(0.4,0.4,0.4,1)
   else
    self.Lbtn.tex:SetVertexColor(1,1,1,1)
   end
   
   if pages:Curr(opened_list) == pages:Max(opened_list) then
    self.Rbtn.tex:SetVertexColor(0.4,0.4,0.4,1)
   else
    self.Rbtn.tex:SetVertexColor(1,1,1,1)
   end
  end

 f = constructors:CreateStaticBtn(f_preview, tx_rempage, {"BOTTOM", f_preview, "BOTTOM", -20, 10}, {L["del_page"]})
 f:SetScript("OnMouseUp", del_page)
 
 f = constructors:CreateStaticBtn(f_preview, tx_addpage, {"BOTTOM", f_preview, "BOTTOM", 20, 10}, {L["new_page"]})
 f:SetScript("OnMouseUp", new_page)
 
 -- Close button
 f = constructors:CreateListButton(editframe_list)
 f:SetText(BACK)
 f:SetSize(100, 50)
 f:SetPoint("BOTTOM", editframe_list, "BOTTOM", 0, 10)
 f.clickfunc = function()
  editframe_list:Hide()
  editframe:Show()
 end
 editframe_list.btnclose = f

 f = constructors:CreateListButton(editframe_list)
 f:SetText("X")
 f:SetSize(30, 30)
 f:SetPoint("TOPRIGHT")
 f.clickfunc = editframe_list.btnclose.clickfunc
 editframe_list.btnclose2 = f

 editframe.sfQuests = f_quests
 f_quests:SetView("Quests")
 sfColumnClick(f_quests.columns[5])
 
 return editframe_list
end